@@ -9,11 +9,11 @@ import copy
 import time
 
 try:
-    import moves
-except ImportError:
     from . import moves
+except ImportError:
+    import moves
 
-VERSION = '0.0.5'
+VERSION = '0.0.9'
 
 class Ansi:
     """ The terminal ansi chars """
@@ -22,6 +22,7 @@ class Ansi:
     RED = '\033[31m'
     RESET = '\033[0m'
     GRAY = '\033[37m'
+    CYAN = '\033[96m'
 
     @staticmethod
     def disable():
@@ -30,6 +31,7 @@ class Ansi:
         Ansi.RED = ''
         Ansi.RESET = ''
         Ansi.GRAY = ''
+        Ansi.CYAN = ''
 
 class Piece:
     """ Each piece in the chess board """
@@ -70,11 +72,18 @@ class Piece:
         x = src[0]
         y = src[1]
         result = []
-        if self.name == 'pawn':
-            result = moves.pawn_move(self, game, src, dst)
+        if self.name == Piece.PAWN:
+            result = moves.pawn_move(self, game, src)
+        elif self.name == Piece.ROCK:
+            result = moves.rock_move(self, game, src)
+        elif self.name == Piece.QUEEN:
+            result = [*moves.rock_move(self, game, src)]
+        elif self.name == Piece.KING:
+            result = moves.king_move(self, game, src)
+        elif self.name == Piece.KNIGHT:
+            result = moves.knight_move(self, game, src)
         else:
-            if not return_locations:
-                return True
+            result = []
         if return_locations:
             return result
         if dst in result:
@@ -103,6 +112,13 @@ class Game:
         # each cell location be in this list, will be highlighted in rendering
         self.highlight_cells = []
 
+        # this item determines the selected piece using `s` command
+        self.selected_cell = None
+
+        # the player names
+        self.white_player = 'White'
+        self.black_player = 'Black'
+
         # initialize the board
         self.board = []
         for i in range(8):
@@ -172,12 +188,32 @@ class Game:
         cmd_parts = cmd.split()
 
         self.highlight_cells = []
+        self.selected_cell = None
 
         invalid_msg = 'Invalid Command!'
 
         result_msg = 'Runed'
 
-        if len(cmd_parts) == 2:
+        if len(cmd_parts) == 1:
+            if cmd_parts[0] == 'back':
+                if not self.logs:
+                    invalid_msg = 'Please move something first!'
+                else:
+                    # back
+                    new_game = Game()
+                    while self.logs:
+                        if not self.logs[-1].startswith('m'):
+                            self.logs.pop()
+                        else:
+                            self.logs.pop()
+                            break
+                    for cmd in self.logs:
+                        new_game.run_command(cmd)
+                    self.logs = new_game.logs
+                    self.board = new_game.board
+                    self.turn = new_game.turn
+                    return 'OK! now you are one step back!'
+        elif len(cmd_parts) == 2:
             # s <location>
             if cmd_parts[0] == 's':
                 location = cmd_parts[1].replace('.', '-').split('-')
@@ -197,6 +233,7 @@ class Game:
 
                         # show allowed moves
                         self.highlight_cells = allowed_moves
+                        self.selected_cell = location
 
                         return '' if self.highlight_cells else 'This piece cannot move!'
                     except:
@@ -251,12 +288,74 @@ class Game:
 
         return result_msg
 
+    def get_dead_items(self):
+        """ Returns list of dead items like this:
+        {
+            "black": {
+                "pawn": 3, # count
+                "rock": 1
+            },
+            "white": {
+                # ...
+            }
+        }
+        """
+        result = {
+            "white": {},
+            "black": {},
+        }
+        for i in range(0, 8):
+            for j in range(0, 8):
+                if self.board[i][j] is not None:
+                    try:
+                        result[self.board[i][j].color][self.board[i][j].name] += 1
+                    except:
+                        result[self.board[i][j].color][self.board[i][j].name] = 1
+
+        # now, `result` contains live items.
+        # we want dead items
+        for team in result:
+            for item in result[team]:
+                if item == Piece.PAWN:
+                    result[team][item] = 8 - result[team][item]
+                elif item not in (Piece.QUEEN, Piece.KING):
+                    result[team][item] = 2 - result[team][item]
+                else:
+                    result[team][item] = 1 - result[team][item]
+
+        # delete items contains 0
+        new_result = {
+            "white": {},
+            "black": {},
+        }
+        for team in result:
+            for item in result[team]:
+                if result[team][item] > 0:
+                    new_result[team][item] = result[team][item]
+
+        return new_result
+
     def render(self) -> str:
         """ Renders the board to show in the terminal """
         output = ''
+
+        # render the player names
+        white_player = 'B. ' + self.white_player
+        black_player = 'W. ' + self.black_player
+        white_space_len = (len(self.ROW_SEPARATOR) - (len(white_player)+len(black_player))) - 2
+        white_space_len = int(white_space_len/2)
+        player_names = Ansi.CYAN + white_player + Ansi.RESET + (' ' * white_space_len) + 'Vs' + (' ' * white_space_len) + Ansi.RED + black_player + Ansi.RESET
+
+        output += player_names + '\n'
+        output += ('_' * len(self.ROW_SEPARATOR)) + '\n'
+        output += (' ' * len(self.ROW_SEPARATOR)) + '\n'
+
+        for i in range(1, 9):
+            output += (int(self.CELL_WIDTH/2) * ' ') + str(i) + (' ' * (int(self.CELL_WIDTH/2)+1))
+        output += '\n'
         i = 0
         for row in self.board:
-            output += self.ROW_SEPARATOR
+            output += '  ' + self.ROW_SEPARATOR + str(i+1) + ' '
             j = 0
             for column in row:
                 if column is None:
@@ -265,15 +364,41 @@ class Game:
                     ansi_reset = Ansi.RESET
                 else:
                     column_str = str(column)
-                    ansi_color = Ansi.GREEN if column.color == 'white' else Ansi.RED
+                    ansi_color = Ansi.CYAN if column.color == 'white' else Ansi.RED
                     ansi_reset = Ansi.RESET
-                if [i, j] in self.highlight_cells:
-                    column_str = ' *' + column_str.lstrip() + '*'
+                if [i, j] in self.highlight_cells or self.selected_cell == [i, j]:
+                    column_str = '*' + column_str.lstrip() + '*'
                 output += '| ' + ansi_color + column_str + ansi_reset + (' ' * (self.CELL_WIDTH-len(column_str)))
                 j += 1
             output += '|\n'
             i += 1
-        output += self.ROW_SEPARATOR
+        output += '  ' + self.ROW_SEPARATOR
+        lines = output.splitlines()
+        output = ''
+        for line in lines:
+            output += ' ' + line + '\n'
+
+        # show dead items
+        dead_items = self.get_dead_items()
+        dead_items_str = 'Deads:\n'
+        for team in dead_items:
+            if dead_items[team]:
+                color = (Ansi.CYAN if team == 'white' else Ansi.RED)
+                current_line = color + team + Ansi.RESET + ': '
+                for item in dead_items[team]:
+                    if item in (Piece.QUEEN, Piece.KING):
+                        current_line += color + item + Ansi.RESET + ', '
+                    else:
+                        current_line += color + str(dead_items[team][item]) + ' ' + item + Ansi.RESET + ', '
+                current_line = current_line.strip().strip(',')
+                current_line += (' ' * (len(self.ROW_SEPARATOR) - len(current_line)))
+                dead_items_str += current_line + '\n'
+
+        if dead_items_str == 'Deads:\n':
+            dead_items_str = ''
+
+        output += dead_items_str.strip()
+
         return output
 
 def show_help():
@@ -294,6 +419,9 @@ OPTIONS
     --no-ansi: disable terminal ansi colors
     --replay: play the saved game
     --replay-speed: delay between play frame (for example `3`(secound) or `0.5`)
+    --dont-check-terminal: do not check terminal size
+    --player-white=[name]: set name of white player
+    --player-black=[name]: set name of black player
 
 AUTHOR
     This software is created by Parsa Shahmaleki <parsampsh@gmail.com>
@@ -352,16 +480,17 @@ def run(args=[]):
             break
 
     # check the terminal size
-    try:
-        terminal_width = os.get_terminal_size().columns
-    except:
-        terminal_width = len(Game.ROW_SEPARATOR)
-    if terminal_width < len(Game.ROW_SEPARATOR):
-        print(
-            'ERROR: your terminal width is less than ' + str(len(Game.ROW_SEPARATOR)) + '.',
-            file=sys.stderr
-        )
-        sys.exit(1)
+    if not '--dont-check-terminal' in options:
+        try:
+            terminal_width = os.get_terminal_size().columns
+        except:
+            terminal_width = len(Game.ROW_SEPARATOR)+3
+        if terminal_width < len(Game.ROW_SEPARATOR)+3:
+            print(
+                'ERROR: your terminal width is less than ' + str(len(Game.ROW_SEPARATOR)+3) + '. use --dont-check-terminal to ignore it.',
+                file=sys.stderr
+            )
+            sys.exit(1)
 
     if len(arguments) > 0:
         game_file_name = arguments[0]
@@ -391,13 +520,22 @@ def run(args=[]):
     if is_play:
         game = Game()
 
+    # set player names
+    for option in options:
+        if option.startswith('--player-white='):
+            game.white_player = option.split('=', 1)[-1]
+        elif option.startswith('--player-black='):
+            game.black_player = option.split('=', 1)[-1]
+
     # last result of runed command
     last_message = ''
 
     while True:
         # render the game board on the terminal
         print('\033[H', end='')
-        title = '*** Welcome to the TChess! ***'
+        title = ' Welcome to the TChess! '
+        stars_len = len(Game.ROW_SEPARATOR) - len(title)
+        title = (int(stars_len/2) * '*') + title + (int(stars_len/2) * '*')
         print(title, end='')
         print(' ' * (len(Game.ROW_SEPARATOR) - len(title)))
         print(' ' * len(Game.ROW_SEPARATOR))
@@ -405,7 +543,7 @@ def run(args=[]):
 
         # get command from user and run it
         tmp_turn = game.turn
-        ansi_color = Ansi.RED if tmp_turn == 'black' else Ansi.GREEN
+        ansi_color = Ansi.RED if tmp_turn == 'black' else Ansi.CYAN
         # fix whitespace
         print(last_message, end='')
         print(' ' * (len(Game.ROW_SEPARATOR)-len(last_message)))
@@ -422,6 +560,7 @@ def run(args=[]):
             command = input(ansi_color + game.turn + Ansi.RESET + ' Turn >>> ').strip().lower()
 
         game.highlight_cells = []
+        game.selected_cell = None
 
         # check the empty command
         if command == '':
